import unittest

from office.api.markdown import markdown_link_image_to_base64, check_local_dir_image_link_markdown


class TestMarkdown(unittest.TestCase):
    def test_markdown_link_image_to_base64(self, ):
        markdown_link_image_to_base64(
            markdown_path=r"C:\Users\37386\PycharmProjects\python-office\testfile\markdown\test.md")

    def test_check_local_dir_image_link_markdown(self):
        check_local_dir_image_link_markdown(
            markdown_path=r"C:\Users\37386\PycharmProjects\python-office\testfile\markdown\test.md",
            image_path=r"C:\Users\37386\PycharmProjects\python-office\testfile\markdown\test.assets")



import unittest

from office.api.file import file_name_add_prefix, search_specify_type_file, file_name_insert_content, \
    file_name_add_postfix, output_file_list_to_excel


class TestFile(unittest.TestCase):
    def test_file_name_add_prefix(self):
        file_name_add_prefix(file_path=r'D:\workplace\code\test\output\test', prefix_content='2022')

    def test_search_specify_type_file(self):
        search_specify_type_file(file_path=r'test_files/pdf', file_type='.pdf')

    def test_file_name_insert_content(self):
        file_name_insert_content(file_path=r"C:\Users\37386\PycharmProjects\python-office\testfile\file",
                                 insert_position=1, insert_content="插入内容测试")

    def test_file_name_add_postfix(self):
        file_name_add_postfix(r"C:\Users\37386\PycharmProjects\python-office\testfile\file",
                              "添加后缀测试")

    def test_output_file_list_to_excel(self):
        output_file_list_to_excel("../testfile")


from fpdf import FPDF
from office.lib.pdf import add_watermark_service
import pikepdf
from PyPDF2 import PdfFileReader, PdfFileWriter
from pdf2docx import Converter
import os
from pathlib import Path
import fitz  # fitz就是pip install PyMuPDF

from tqdm import tqdm


class MainPDF():

    def add_watermark(self):
        pdf_file_in = input("请输入需要添加水印的文件位置：")  # 需要添加水印的文件
        Watermark_Str = input("请输入需要添加的水印内容：")
        print('=' * 20)
        print('正在按要求，给你的PDF文件添加水印，请让程序飞一会儿~')
        print('=' * 20)
        pdf_file_mark = 'watermark.pdf'  # 水印文件
        add_watermark_service.create_watermark(str(Watermark_Str))
        pdf_file_out = '添加了水印的文件.pdf'  # 添加PDF水印后的文件
        add_watermark_service.pdf_add_watermark(pdf_file_in, pdf_file_mark, pdf_file_out)
        print("水印添加结束，请打开电脑上的这个位置，查看结果文件：{path}".format(path=os.getcwd()))

    def add_watermark_by_parameters(self, pdf_file, mark_str, output_file_name):
        """
        给pdf添加水印，需要参数的版本
        """
        # pdf_file_in = input("请输入需要添加水印的文件位置：")  # 需要添加水印的文件
        # Watermark_Str = input("请输入需要添加的水印内容：")
        print('=' * 20)
        print('正在按要求，给你的PDF文件添加水印，请让程序飞一会儿~')
        print('=' * 20)
        pdf_file_mark = 'watermark.pdf'  # 水印文件
        add_watermark_service.create_watermark(str(mark_str))
        pdf_file_out = output_file_name  # '添加了水印的文件.pdf'  # 添加PDF水印后的文件
        add_watermark_service.pdf_add_watermark(pdf_file, pdf_file_mark, pdf_file_out)
        print("水印添加结束，请打开电脑上的这个位置，查看结果文件：{path}".format(path=os.getcwd()))

    def file2pdf(self, file_type, path, res_pdf='file2pdf.pdf'):
        if file_type == 'txt':
            pdf = FPDF()
            pdf.add_page()  # Add a page
            pdf.set_font("Arial", size=15)  # set style and size of font
            f = open(path, "r")  # open the text file in read mode
            # insert the texts in pdf
            for x in f:
                pdf.cell(50, 5, txt=x, ln=1, align='C')
            # pdf.output("path where you want to store pdf file\\file_name.pdf")
            pdf.output(res_pdf)

    def pdf2docx(self, file_path, output_path):
        word_name = os.path.basename(file_path)[:-4] + '.docx'
        word_path = Path(output_path) / word_name
        cv = Converter(file_path)
        cv.convert(word_path)
        cv.close()

    # 合并pdf
    def merge2pdf(self, one_by_one, output):
        """
        @Author & Date  : CoderWanFeng 2022/5/16 23:33
        @Desc  : merge_pdfs(paths=['开篇词.pdf', '中国元宇宙白皮书 (送审稿).pdf'], output='merge.pdf')
        """
        pdf_writer = PdfFileWriter()

        for path in one_by_one:
            pdf_reader = PdfFileReader(path)
            for page in tqdm(range(pdf_reader.getNumPages())):
                # 把每张PDF页面加入到这个可读取对象中
                pdf_writer.addPage(pdf_reader.getPage(page))

        # 把这个已合并了的PDF文档存储起来
        with open(output, 'wb') as out:
            pdf_writer.write(out)

    # PDF加密
    def encrypt4pdf(self, path, password, res_pdf='encrypt.pdf'):
        """
        @Author & Date  : CoderWanFeng 2022/5/9 18:27
        @Desc  : path: 存放文件的路径
                password: 你的密码
                res_pdf: 结果文件的名称 ，可以为空，默认是：encrypt.pdf
        """
        pdf = pikepdf.open(path)
        pdf.save(res_pdf, encryption=pikepdf.Encryption(owner=password, user=password, R=4))
        pdf.close()

    # PDF解密
    def decrypt4pdf(self, path, password, res_pdf='decrypt.pdf'):
        pdf = pikepdf.open(path, password=password)
        pdf.save(res_pdf)
        pdf.close()

    # def pdf2imgs(self, pdf_path: str, out_dir=".") -> None:
    #     print('PDF开始转换，你可以加入交流群唠唠嗑：http://www.python4office.cn/wechat-group/')
    #     pdfDoc = fitz.open(pdf_path)
    #     if pdfDoc.pageCount > 50:
    #         print('少年，你的PDF页数有点多哟，请耐心等待~')
    #     for pg in range(pdfDoc.pageCount):
    #         page = pdfDoc[pg]
    #         rotate = int(0)
    #         # 每个尺寸的缩放系数为1.3，这将为我们生成分辨率提高2.6的图像。
    #         # 此处若是不做设置，默认图片大小为：792X612, dpi=96
    #         zoom_x = 1.33333333  # (1.33333333-->1056x816)   (2-->1584x1224)
    #         zoom_y = 1.33333333
    #         mat = fitz.Matrix(zoom_x, zoom_y).preRotate(rotate)
    #         pix = page.getPixmap(matrix=mat, alpha=False)

    #         if not os.path.exists(out_dir):  # 判断存放图片的文件夹是否存在
    #             os.makedirs(out_dir)  # 若图片文件夹不存在就创建

    #         pix.writePNG(out_dir + '/' + 'images_%s.png' % pg)  # 将图片写入指定的文件夹内
    #     print(f'PDF转换Image完成，图片在你指定的output文件夹{out_dir}，如果没有指定，默认是PDF同一个文件夹')

    def pdf2imgs(self, pdf_path: str, out_dir=".") -> None:
        print('PDF开始转换，你可以加入交流群唠唠嗑：http://www.python4office.cn/wechat-group/')
        pdfDoc = fitz.open(pdf_path)
        if pdfDoc.page_count > 50:
            print('少年，你的PDF页数有点多哟，请耐心等待~')
        for pg in tqdm(range(pdfDoc.page_count)):
            page = pdfDoc[pg]
            rotate = int(0)
            # 每个尺寸的缩放系数为1.3，这将为我们生成分辨率提高2.6的图像。
            # 此处若是不做设置，默认图片大小为：792X612, dpi=96
            zoom_x = 1.33333333  # (1.33333333-->1056x816)   (2-->1584x1224)
            zoom_y = 1.33333333
            mat = fitz.Matrix(zoom_x, zoom_y).prerotate(rotate)
            pix = page.get_pixmap(matrix=mat, alpha=False)

            if not os.path.exists(out_dir):  # 判断存放图片的文件夹是否存在
                os.makedirs(out_dir)  # 若图片文件夹不存在就创建

            pix.save(out_dir + '/' + 'images_%s.png' % pg)  # 将图片写入指定的文件夹内
        print(f'PDF转换Image完成，图片在你指定的output文件夹{out_dir}，如果没有指定，默认是PDF同一个文件夹')

    def add_img_watermark(self, pdf_file_in, pdf_file_mark, pdf_file_out):
        add_watermark_service.pdf_add_watermark(pdf_file_in, pdf_file_mark, pdf_file_out)



from office.lib.utils.except_utils import except_dec

from office.core.ToolsType import MainTools

mainTools = MainTools()


@except_dec()
def transtools(to_lang, content):
    mainTools.transtools(to_lang, content)


@except_dec()
def qrcodetools(url):
    mainTools.qrcodetools(url)


@except_dec()
def passwordtools(len=8):
    mainTools.passwordtools(len)


@except_dec()
def weather():
    mainTools.weather()


# 通过url，获取ip地址
# @except_dec()
def url2ip(url):
    mainTools.url2ip(url)


# 通过url，获取ip地址
@except_dec()
def lottery8ticket():
    mainTools.lottery8ticket()


@except_dec()
def create_article(theme, line_num=200):
    mainTools.create_article(theme, line_num)


@except_dec()
def pwd4wifi(len_pwd=8, pwd_list=[]):
    mainTools.pwd4wifi(len_pwd,pwd_list)



import base64
import pathlib


class MainMarkdown():

    def markdown_link_image_to_base64(self, markdown_path):
        markdown_path = pathlib.Path(markdown_path)
        if markdown_path.is_file() and markdown_path.suffix == ".md":
            i = 0  # 图片计数变量
            image_base64_list = []  # 图片base64编码列表
            markdown_file = open(markdown_path, encoding="utf-8")
            new_markdown_file = open(markdown_path.parent.joinpath(markdown_path.name[:-3:] + "(base64).md"), "x",
                                     encoding="utf-8")
            markdown_file = markdown_file.readlines()
            for line in markdown_file:
                # 按行遍历文件
                if line[:2:] == "![":
                    # 如果该行有链接图片
                    i = i + 1
                    this_image_path = pathlib.Path(line[line.rfind("(") + 1:line.rfind(")"):])
                    if not this_image_path.is_absolute():
                        # Python文件和markdown文件通常不在同一路径，相对路径基准不同，故需要判断链接图片路径是否为绝对路径。
                        this_image_path = markdown_path.parent.joinpath(this_image_path).resolve()
                    this_image_file = open(this_image_path, "rb")
                    this_image_base64_data = base64.b64encode(this_image_file.read())
                    # 对链接的图片进行base64编码
                    line = line[:line.index("("):] + f"[image{str(i)}]" + "\n"
                    image_base64_list.append(
                        f"[image{str(i)}]: " + "data:image/" + this_image_path.suffix + ";base64," + str(
                            this_image_base64_data)[2:-1:] + "\n")
                new_markdown_file.writelines(line)
            for i in image_base64_list:
                # 遍历存放base64编码的列表
                if markdown_file[-1] != "\n":
                    new_markdown_file.writelines("\n")
                # 判断最后一行是否为空行，如果不是则写入换行符
                new_markdown_file.writelines(i)
                # 将图片的base64编码写入新文件
        elif not markdown_path.is_file():
            print("markdown文件路径输入有误，请检查！")
        elif markdown_path.suffix != ".md":
            print("请输入.md文件的路径！")

    def check_local_dir_image_link_markdown(self, markdown_path, image_path):
        """
        :param markdown_path: markdown文件路径
        :param image_path: 本地图片存放路径
        """
        markdown_path = pathlib.Path(markdown_path)
        image_path = pathlib.Path(image_path)
        # 转换路径为对象
        if markdown_path.is_file() and image_path.is_dir():
            print("检查中……")
            markdown_image_list = []  # markdown文件中链接的图片列表
            markdown_file = open(markdown_path, encoding="utf-8")
            markdown_file = markdown_file.readlines()
            for line in markdown_file:
                # 按行遍历文件
                if line[:2:] == "![":
                    # 如果该行有链接图片
                    this_image_path = pathlib.Path(line[line.rfind("(") + 1:line.rfind(")"):])
                    # 提取链接图片路径
                    if this_image_path.is_absolute():
                        # Python文件和markdown文件通常不在同一路径，相对路径基准不同，故需要判断链接图片路径是否为绝对路径。如果是则直接添加进列表，如果不是则要拼接上markdown所在的路径
                        markdown_image_list.append(this_image_path)
                    else:
                        this_image_path = markdown_path.parent.joinpath(this_image_path).resolve()
                        markdown_image_list.append(this_image_path)
            local_dir_image_list = []
            for i in list(image_path.glob("**/*")):
                local_dir_image_list.append(i.resolve())
            markdown_image_set = set(markdown_image_list)
            local_dir_image_set = set(local_dir_image_list)
            if local_dir_image_set ^ markdown_image_set != (
                    local_dir_image_set ^ markdown_image_set) & local_dir_image_set:
                """
                如果有文件夹中有图片未被链接到markdown中或markdown中链接了其他路径的图片，则local_dir_image_set ^ markdown_image_set不为空
                (local_dir_image_set ^ markdown_image_set) & local_dir_image_set为未被链接到markdown中的图片
                如果两个集合不相等，则代表markdown中链接了其他路径的图片
                """
                print("文档中有链接其他路径的图片，请检查！ ")
                other_dir_image_list = list((local_dir_image_set ^ markdown_image_set) & markdown_image_set)
                print("其他路径图片列表：")
                for i in other_dir_image_list:
                    print(str(i))
                print()
            if (local_dir_image_set ^ markdown_image_set) & local_dir_image_set != set():
                print("未被链接图片列表：")
                for i in local_dir_image_set ^ markdown_image_set & local_dir_image_set:
                    # 打印未被链接图片名列表
                    print(i)
            else:
                print(f"{image_path.resolve()}内无未被链接的图片")
        elif not markdown_path.is_file():
            print("markdown文件路径输入有误，请检查！")
        elif not image_path.is_dir():
            print("图片存放路径输入有误，请检查！")



import unittest

from office.api.pdf import *


class TestExcel(unittest.TestCase):
    def test_pdf2imgs(self):
        pdf2imgs(
            pdf_path=r'C:\Users\lenovo\Documents\WeChat Files\wxid_4zuh1m3d6dw212\FileStorage\MsgAttach\f1f9730d6e856d01d0aa5fcba49ea770\File\2022-07\鼎朗互娱_通用版_短视频合作协议.pdf',
            out_dir='./images')

    def test_pdf2docx(self):
        pdf2docx(
            file_path=r'D:\如何利用Python进行自动化办公.pdf',
            output_path=r'C:\output\test'
        )

    def test_add_img_water(self):
        add_img_water(pdf_file_in='test_files/pdf/add_img.pdf', pdf_file_mark='test_files/pdf/in.pdf', pdf_file_out='add_img.pdf')

    def test_add_watermark_by_parameters(self):
        add_watermark_by_parameters(
            pdf_file=r'C:\Users\Lenovo\Documents\WeChat Files\wxid_z91t05fqtrry22\FileStorage\MsgAttach\7ae419aa6a5f7c2fc32c594a69e28c6d\File\2022-06\2022眼科行业研究报告-动脉网-2022-52页.pdf',
            mark_str='abc',
            output_file_name='测试.pdf')



import unittest
from pathlib import Path

import pandas as pd
from office.api.excel import fake2excel, find_excel_data, split_excel_by_column, sheet2excel
import os


class TestExcel(unittest.TestCase):
    def test_fake2excel(self):
        fake2excel(language='fdsa')

    def test_merge2sheet(self):
        """
        https://blog.csdn.net/xue_11/article/details/118424380
        https://www.jb51.net/article/214868.htm
        """
        dir_path = 'test_files/excel'
        for root, dirs, files in os.walk(dir_path):
            path = Path(dir_path)
            print(files)
            df_list = []
            for file in files:
                if file.endswith("xlsx") or file.endswith("xls"):
                    excel_path = (path / file)
                    df_list.append(pd.read_excel(excel_path))
            res = pd.concat(df_list)
            res.to_excel(
                R"./excel/output_file2.xlsx",
                sheet_name="手机商品",
                index=False  # 不保留index
            )

        # single_df_1 = pd.read_excel(r'./excel/1月.xls')
        # print(single_df_1)
        # single_df_2 = pd.read_excel(r'./excel/2月.xls')
        # single_df_3 = pd.read_excel(r'./excel/3月.xls')
        # single_df_4 = pd.read_excel(r'./excel/4月.xls')
        # res = pd.concat([single_df_1, single_df_2,single_df_3,single_df_4])
        # res.to_excel(
        #     R"./excel/output_file1.xlsx",
        #     sheet_name="手机商品",
        #     index=False,  # 不保留index
        # )

    def test_find_excel_data(self):
        find_excel_data(search_key='刘家站垦殖场', target_dir=r'D:\workplace\code\gitee\python-office\contributors\bulabean')

    def test_split_excel_by_column(self):
        split_excel_by_column(filepath=r'D:\workplace\code\gitee\python-office\contributors\bulabean\sedemo.xls',
                              column=6)

    def test_sheet2excel(self):
        sheet2excel(file_path=r'/tests/excel/fake2excel.xlsx')



from office.core.MarkdownType import MainMarkdown
from office.lib.utils.except_utils import except_dec

mainMarkdown = MainMarkdown()


@except_dec()
def markdown_link_image_to_base64(markdown_path):
    mainMarkdown.markdown_link_image_to_base64(markdown_path)


@except_dec()
def check_local_dir_image_link_markdown(markdown_path, image_path):
    mainMarkdown.check_local_dir_image_link_markdown(markdown_path, image_path)